#include "shader.h"
#include "types.h"
#include "resource.h"

#include <GL/glew.h>

namespace kek {

static GLuint compileShader(GLenum type, std::string path) {
	MemoryBuffer *buf = Resource::loadResource(path);
	if(!buf) {
		std::cout << "Couldn't load shader: " << path << std::endl;
		throw std::exception();
	}

	std::ostringstream stream;
	stream << buf;
	std::string code = stream.str();

	delete buf;

	const char *src = code.c_str();

	GLint success;

	GLuint vertex = glCreateShader(type);
	glShaderSource(vertex, 1, &src, NULL);
	glCompileShader(vertex);

	glGetShaderiv(vertex, GL_COMPILE_STATUS, &success);
	if(!success) {
		char log[512];
		glGetShaderInfoLog(vertex, 512, NULL, log);
		std::cout << "Failed to compile vertex shader \"" << path << "\":\n" << log << std::endl;
	}

	return vertex;
}

static GLuint compileProgram(unsigned int n, GLenum *types, std::string *paths) {
	// Compile shaders
	GLuint shaderIds[n];
	for(unsigned int i = 0; i < n; i++) {
		shaderIds[i] = compileShader(types[i], paths[i]);
	}

	GLuint id = glCreateProgram();

	// Attach shaders
	for(unsigned int i = 0; i < n; i++) {
		glAttachShader(id, shaderIds[i]);
	}

	glLinkProgram(id);

	GLint success;
	glGetProgramiv(id, GL_LINK_STATUS, &success);

	if(!success) {
		char log[512];
		glGetProgramInfoLog(id, 512, NULL, log);
		std::cout << "Failed to link program:\n" << log << std::endl;
	}

	// Delete shaders
	for(unsigned int i = 0; i < n; i++) {
		glDeleteShader(shaderIds[i]);
	}

	return id;
}

Shader::Shader(std::string vertexPath, std::string fragmentPath) {

}

Shader::Shader(std::string vertexPath, std::string geometryPath, std::string fragmentPath) {

}

Shader::~Shader() {

}

void Shader::use() {

}

}